package com.jobs.test;

import com.jobs.pojo.Company;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.geo.*;
import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.geo.GeoJsonPoint;
import org.springframework.data.mongodb.core.geo.GeoJsonPolygon;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.NearQuery;
import org.springframework.data.mongodb.core.query.Query;

import java.util.ArrayList;
import java.util.List;

@SpringBootTest
public class MongoGeoTest {

    //注入 spring-data-mongodb 自带的 MongoTemplate 对象
    @Autowired
    private MongoTemplate mongoTemplate;

    //添加空间测试的样例数据
    @Test
    public void addSampleTestData() {

        List<Company> companyList = new ArrayList<>();
        companyList.add(new Company().setName("任肥肥洗浴中心")
                .setLocation(new double[]{116.409488, 39.917015}));
        companyList.add(new Company().setName("侯胖胖足疗按摩")
                .setLocation(new double[]{116.39818, 39.918844}));
        companyList.add(new Company().setName("王棒棒皮包中介")
                .setLocation(new double[]{116.411615, 39.921669}));
        companyList.add(new Company().setName("李墩墩娱乐会所")
                .setLocation(new double[]{116.409229, 39.920868}));
        companyList.add(new Company().setName("范呆呆赝品批发")
                .setLocation(new double[]{116.3949, 39.97528}));
        companyList.add(new Company().setName("蔺赞赞快餐连锁")
                .setLocation(new double[]{116.398053, 39.920301}));
        companyList.add(new Company().setName("杨壮壮二婚交友")
                .setLocation(new double[]{116.412929, 39.917812}));
        companyList.add(new Company().setName("乔豆豆下岗培训")
                .setLocation(new double[]{116.409182, 39.921745}));

        //批量添加 8 条样例数据
        mongoTemplate.insert(companyList, Company.class);
    }

    //给定一个中心点，查询 800 米以内的公司
    @Test
    public void testCircle() {
        //指定中心点
        GeoJsonPoint point = new GeoJsonPoint(116.404, 39.915);
        //KILOMETERS 表示千米
        Distance distance = new Distance(0.8, Metrics.KILOMETERS);
        //构建一个圆形范围
        Circle circle = new Circle(point, distance);
        //构造查询条件
        Query query = Query.query(Criteria.where("location").withinSphere(circle));
        //5、查询
        List<Company> list = mongoTemplate.find(query, Company.class);
        list.forEach(System.out::println);
    }

    //查询距离给定的中心点 800 米以内的公司，并按照距离，由近到远进行排列。
    @Test
    public void testNearest() {
        //指定中心点
        GeoJsonPoint point = new GeoJsonPoint(116.404, 39.915);
        //指定查询条件
        NearQuery query = NearQuery.near(point).maxDistance(new Distance(0.8, Metrics.KILOMETERS));
        //由于 Company 的字段 location 设置了空间索引，因此针对该字段与给定点进行距离计算并筛选。
        GeoResults<Company> results = mongoTemplate.geoNear(query, Company.class);
        //打印查询结果，按照距离给定点的距离由近到远排列
        for (GeoResult<Company> result : results) {
            Company company = result.getContent();
            double value = result.getDistance().getValue();
            System.out.println(company.getName() + " 距离给定点：" + (int) (value * 1000) + " 米");
        }
    }

    //查询自定义多边形区域内的公司
    @Test
    public void testRectangle() {
        //构建一个三角形（需要注意：多边形首尾两个点的坐标必须相同）
        List<Point> points = new ArrayList<>();
        points.add(new Point(116.361, 39.924));
        points.add(new Point(116.415, 39.933));
        points.add(new Point(116.393, 39.891));
        points.add(new Point(116.361, 39.924));

        GeoJsonPolygon polygon = new GeoJsonPolygon(points);
        Query query = Query.query(Criteria.where("location").within(polygon));
        List<Company> list = mongoTemplate.find(query, Company.class);
        list.forEach(System.out::println);
    }
}
